#!/usr/bin/python
# -*- coding: UTF-8 -*-
from dli.column import Column

from dli.exception import DliException
from dli.rest import ExportCommonInfo
from dli.rest.rest import ApiException
from dli.result_set import ResultSet
from dli.row import Row


class SqlJob(object):
    def __init__(self, job_id, job_type, client, schema=None, rows=None, status=None):
        """

        :param str job_id:
        :param str job_type:
        :param DliClient client:
        :param list schema:
        :param list rows:
        """

        self._job_id = job_id
        self._job_type = job_type
        self._schema = []
        self._rows = []
        self._client = client
        self._status = status

        if schema is not None:
            for col in schema:
                for col_name, col_type in col.items():
                    self._schema.append(Column(col_name, col_type))

        if rows is not None and schema is not None:
            for row in rows:
                self._rows.append(Row(self._schema, row))

    @property
    def job_id(self):
        return self._job_id

    @property
    def job_type(self):
        return self._job_type

    @property
    def job_status(self):
        return self._status

    def get_result_page(self, current_page):
        kwargs = {
            'project_id': self._client.dli_info.project_id,
            'x_auth_token': self._client.token,
            'job_id': self.job_id,
            'page_size': 100,
            'current_page': current_page
        }

        try:
            resp = self._client.rest_api.get_job_result(**kwargs)
        except ApiException as e:
            raise DliException(api_exception=e)
        if not resp.is_success:
            raise DliException(msg=resp.message)

        return resp

    def get_schema(self):
        """
        This method is used to view the job execution result after a job
        is executed using SQL query statements. Currently, you can only
        query execution results of jobs of the QUERY type.
        You can view the first 1000 result records only. To view all result
        records, use export_result method.

        :return: list: The list of Columns, have the name and type of the job result column.
        """

        # Sql is DCL, DDL or Query in sync mode, the result rows have got
        # from submit job API call
        if len(self._schema) == 0 and self.job_type == 'QUERY':
            resp = self.get_result_page(1)
            for col in resp.schema:
                for col_name, col_type in col.items():
                    self._schema.append(Column(col_name, col_type))

        return self._schema

    def get_result(self):
        """
        This method is used to view the job execution result after a job
        is executed using SQL query statements. Currently, you can only
        query execution results of jobs of the QUERY type.
        When the query returns more records, it is recommended to use export_result method.

        :return: ResultSet.
        """

        # Sql is DCL, DDL the result rows have got
        # from submit job API call
        if self.job_type == 'QUERY':
            return ResultSet(self)
        else:
            return ResultSet(self, self._rows)

    def export_result(self, data_path, queue_name='default', compress='none',
                      export_mode='ErrorIfExists'):
        """
        This method is used to export results returned from the query
        using SQL statements to OBS.

        :param str data_path: Path for storing the exported result (currently,
                        data can be stored on an OBS path starting with s3a only). The
                        OBS path cannot contain folders, for example, the path folder
                        in the sample request.
        :param str queue_name: Name of the queue that is specified to execute a task.
                        If no queue is specified, the default queue is used.
        :param str compress: Compression format of exported data. Currently, gzip,
                        bzip2, and deflate are supported. The default value is
                        none, indicating that data is not compressed.
        :param str export_mode: Export mode. The parameter value can be
                        ErrorIfExists or Overwrite. If export_mode is not
                        specified, this parameter is set to ErrorIfExists by default.
        :return: str Status of a job, including LAUNCHING, RUNNING,
                        FINISHED, FAILED, and CANCELLED.
        """

        exp_info = ExportCommonInfo(data_path=data_path, data_type='csv',
                                    compress=compress, queue_name=queue_name,
                                    export_mode=export_mode)
        kwargs = {
            'project_id': self._client.dli_info.project_id,
            'x_auth_token': self._client.token,
            'job_id': self.job_id,
            'export_common_info': exp_info
        }

        try:
            resp = self._client.rest_api.store_job_result(**kwargs)
        except ApiException as e:
            raise DliException(api_exception=e)
        if not resp.is_success:
            raise DliException(msg=resp.message)

        return self._client.wait_for_finish(resp.job_id)
